<?php
class XEventManager {
    private static $log_rules = [
        'register' => false,
        'remove' => false,
    ];

    protected static $events = [];
    protected static $listeners_to_event_index = [];

    public static function on($eventName, $listener, $options = []) {
        if (!isset(self::$events[$eventName])) {
            self::$events[$eventName] = [];
        }
        $listenerId = XUtils::guid();
        $listener['_id'] = $listenerId;
        $listener['_options'] = $options;
        self::$events[$eventName][] = $listener;
        self::$listeners_to_event_index[$listenerId] = $eventName;
        if (self::$log_rules['register']) {
            global $logger;
            $logger->log("XEM Register " . $eventName, $listenerId);
        }
        return $listenerId;
    }

    public static function once($eventName, $listener) {
        return self::on($eventName, $listener, ['_once' => true]);
    }

    public static function remove($listenerId) {
        if (isset(self::$listeners_to_event_index[$listenerId])) {
            $eventName = self::$listeners_to_event_index[$listenerId];
            foreach (self::$events[$eventName] as $index => $listener) {
                if ($listener['_id'] == $listenerId) {
                    array_splice(self::$events[$eventName], $index, 1);
                }
            }
            unset(self::$listeners_to_event_index[$listenerId]);
            if (self::$log_rules['remove']) {
                XLogger::log("XEM Remove " . $eventName, $listenerId);
            }
        }
    }

    public static function fire($eventName, $data = null) {
        if (isset(self::$events[$eventName])) {
            $eventsToRemove = [];
            foreach (self::$events[$eventName] as $listener) {
                call_user_func($listener, $data);
                if (isset($listener['_options']['_once']) && $listener['_options']['_once'] && isset($listener['_id'])) {
                    $eventsToRemove[] = $listener['_id'];
                }
            }
            foreach ($eventsToRemove as $listenerId) {
                self::remove($listenerId);
            }
        }
    }
}


?>