<?php

require_once 'XLogger.php';
require_once 'XObject.php';

/**
 * Xpell Module Object Manager
 * @description This manager holds the module XObjects that should be managed (XObject children will not be managed separately)
 * XModules uses the Object Manager to create new XObjects by providing the class of the object by name
 */

class XObjectManager {
    private $_object_classes = [];
    private $_xobjects = [];
    private $_names_index = [];

    public function __construct($moduleName = null) {
        if ($moduleName) {
            // XLogger::log("Object Manager for " . $moduleName . " loaded");
        } else {
            XLogger::log("Object Manager loaded");
        }
    }

    public function get_objects() {
        return $this->_xobjects;
    }

    public function hasObject($xObjectId) {
        return array_key_exists($xObjectId, $this->_xobjects);
    }

    public function registerObjects($xObjects) {
        foreach ($xObjects as $name => $xObject) {
            $this->registerObject($name, $xObject);
        }
    }

    public function registerObject($name, $xObject) {
        $this->_object_classes[$name] = $xObject;
    }

    public function hasObjectClass($name) {
        return array_key_exists($name, $this->_object_classes);
    }

    public function getObjectClass($name) {
        return $this->_object_classes[$name];
    }

    public function getAllClasses() {
        return $this->_object_classes;
    }

    public function get_classes() {
        return $this->_object_classes;
    }

    public function addObject($xObject) {
        if ($xObject && $xObject->_id) {
            $this->_xobjects[$xObject->_id] = $xObject;
            if (!$xObject->_name || strlen($xObject->_name) == 0) {
                $xObject->_name = $xObject->_id;
            }
            $this->_names_index[$xObject->_name] = $xObject->_id;
        } else {
            XLogger::log("unable to add object");
        }
    }

    public function removeObject($xObjectId) {
        $obj = $this->_xobjects[$xObjectId];
        if ($obj) {
            unset($this->_names_index[$obj->_name]);
            unset($this->_xobjects[$xObjectId]);
        }
    }

    public function getObject($xObjectId) {
        return $this->_xobjects[$xObjectId];
    }

    public function go($id) {
        return $this->getObject($id);
    }

    public function getObjectByName($objectName) {
        if (array_key_exists($objectName, $this->_names_index)) {
            return $this->getObject($this->_names_index[$objectName]);
        }
        return null;
    }
}

?>