<?php

class XUtils {
    /**
     * Create ignore list for parser to ignore spells words
     * @param string $list - list of reserved words (comma separated)
     * @param array $reservedWords - reserved words array
     * @return array
     */
    public static function createIgnoreList($list, $reservedWords) {
        $words = explode(",", $list);
        $outList = $reservedWords;
        foreach ($words as $word) {
            $outList[$word] = "";
        }
        return $outList;
    }

    /**
     * Generates GUID (Globally unique Identifier)
     * @return string
     */
    public static function guid() {
        $chars = str_split('0123456789abcdef');
        $uuid = [];
        $rnd = function() { return mt_rand(0, 15); };
        $uuid[8] = $uuid[13] = $uuid[18] = $uuid[23] = '-';
        $uuid[14] = '4'; // version 4
        for ($i = 0; $i < 36; $i++) {
            if (!isset($uuid[$i])) {
                $r = $rnd();
                $uuid[$i] = $chars[($i === 19) ? ($r & 0x3) | 0x8 : $r & 0xf];
            }
        }
        return implode('', $uuid);
    }

    /**
     * Merges XDataObject with Defaults object
     * @param array $data - data of the Xpell command
     * @param array $defaults - defaults object to merge with
     * @param bool $force - add defaults values even if exists
     */
    public static function mergeDefaultsWithData(&$data, $defaults, $force = false) {
        if ($data) {
            if (!isset($data["_id"])) {
                $data["_id"] = isset($data["id"]) ? $data["id"] : self::guid();
            }
            // selective assign
            foreach ($defaults as $key => $value) {
                if (!array_key_exists($key, $data) || $force) {
                    $data[$key] = $value;
                }
            }
        } else {
            $data = $defaults;
        }
    }

    /**
     * Encode string to Base-64 format
     * @param string $str - string to encode
     * @return string
     */
    public static function encode($str) {
        return base64_encode($str);
    }

    /**
     * Decode Base64 String to text
     * @param string $str - Base64 encoded string
     * @return string
     */
    public static function decode($str) {
        return base64_decode($str);
    }

    /**
     * Returns a random integer between min (inclusive) and max (inclusive).
     * The value is no lower than min (or the next integer greater than min
     * if min isn't an integer) and no greater than max (or the next integer
     * lower than max if max isn't an integer).
     * Using Math.round() will give you a non-uniform distribution!
     * @param int $min
     * @param int $max
     * @return int
     */
    public static function getRandomInt($min, $max) {
        $min = ceil($min);
        $max = floor($max);
        return mt_rand($min, $max);
    }

    /**
     * Extracts parameter from XCommand
     * @param XCommand $xcmd - XCommand object
     * @param string $paramName - The name of the parameter to extract
     * @param mixed $defaultValue - Default value if parameter is not found
     * @return mixed
     */
    public static function getParam($xcmd, $paramName, $defaultValue = 0) {
        return isset($xcmd->_params[$paramName]) ? $xcmd->_params[$paramName] : $defaultValue;
    }
}

?>